# Chat Server

Basic Skeleton :This is a node module for a simple chat application. Easy to use readymade functions are available. The chat application is based on socket.io.

## Getting Started

###Step 1:

Install the node module into your node project using the following npm command

```
npm i chat-server-pick-and-play
```

###Step 2:

Place the .env file in the root of the chat-server-pick-and-play node module. The following properties can be set in the .env file
```
DB_HOST, DB_NAME, S3_BUCKET_NAME, S3_ACCESS_KEY, S3_SECRET_KEY, CLOUD_FRONT, HOST_NAME, ACCESS_HOST, ACCESS_PORT, iOS_CERT_PEM_FILE, iOS_KEY_PEM_FILE, PRODUCTION_MODE, IOS_PASSWORD, ANDROID_API_KEY, TWILIO_ACCOUNT_ID, TWILIO_AUTH_TOKEN, TWILIO_FROM_NUMBER, SMS_NUMBER_TO_NOTIFY_ADMIN, SMS_LIMIT, REGISTRATION_TOKEN_EXPIRY, ADMIN_EMAILID, ADMIN_LOGINKEY, ADMIN_USERID, ADMIN_GROUPNAME.
```
Further configurable properties can be found under config folder of the node module

If the .env file is to be placed else where, configure the location of the .env file in the file web/server.js by over writing the line
```
env(__dirname + '/../.env');
```
###Step 3: How to use in your project?
Initialise and require the required modules into your node server as follows

```
var app = express();
var http = require('http').Server(app);
var localClient = require('socket.io-client');
var io = require('socket.io')(http);
var chat = require('chat-server-pick-and-play')(app,io,localClient);
```

Reference links to mandatory modules:

* [Socket-io Client](https://www.npmjs.com/package/socket.io-client)
* [Socket-io](https://www.npmjs.com/package/socket.io)

Link to sample project that uses the Chat Server Pick and Play node module:

[Sample APP](http://gitlab.payoda.com:8888/shinymetilda.J/sample-chat-server-app.git)

##Other Third-Party Libraries Integrated and Available:
###Node Mailer
* [Node Mailer Module Link](https://www.npmjs.com/package/nodemailer2)
Node Mailer will be installed by default. The Node Mailer(Send using a transport plugin) accessKeyId and secretAccessKey can be set in .env file using properties S3_ACCESS_KEY and S3_SECRET_KEY respectively.
This can be used to send notification to the user regarding a new chat room received.
Email templates are rendered using * [ejs node module](https://www.npmjs.com/package/ejs). Further customisations can be made in chat-server-pick-and-play/emailservice/emailservice.js 

###Twillio SMS
* [Twillio SMS Link](https://www.twilio.com/sms)
All Twillio configurations can be set in .env file using the properties TWILIO_ACCOUNT_ID, TWILIO_AUTH_TOKEN.
SMS from numbers provided by Twillio to be used as sender number must be set in node-module/chat-server-pick-and-play/config/smsnumberconfig.js file inside the Array
```
Example:

smsnumberconfig.number = ["+555555555","666666666"];
```


##Rest API's:
The ready made API's available in the chat server pick and play module is elaborated in the swagger docs available in the following link

* [Chat Server Docs](http://gitlab.payoda.com:8888/shinymetilda.J/ChatServerSwaggerAPI.git)

##Socket API's:
###General Sample Syntax:
```
socket.emit('Socket Name', data);
```

###>> Socket Name: adduser

For the user to join a specific room.
Sample Input Data:

```
{"userid":"userid","roomid":"roomid"}
```
###>> Socket Name: sendchat

A client sends a chat message to another, server process it and broadcast to the target.
Sample Input Data:

```
{"userid":"userid","type":"type","roomid":"roomid","username":"username","message":"message"}
```
###>> Socket Name: updatechat

A client can listen to any new chat messages from the server and use this to update messge in a room(Server emit's updatechat as a result of request from sendchat).

Sample Input Data:

```
{"username":"userId","roomid":"roomID","type":"type","message":"message","timestamp":"time"}
```

###>> Socket Name: typeinprogress

It is emitted to a server by a client when a user is typing in a chat room.The server in turn emits this event to all other users in the chat room.
Sample Input Data:

```
{"username":"username", "roomid":"roomid"};
```
###>> Socket Name: typeinstopped

It is emitted to a server by a client when a user has stopped typing in a chat room.The server in turn emits this event to all other users in the chat room.
Sample Input Data:

```
{"username":"username", "roomid":"roomid"};
```
###>> Socket Name: uploadmedia

Used to upload media to S3 bucket.

Sample Input Data:

```
{ "username":"username", "roomid":"roomID", "filetype":"fileType","timestamp":"time", "maxAsyncS3": "maxAsyncS3",    "s3RetryCount": "s3RetryCount", "s3RetryDelay": "s3RetryDelay","multipartUploadThreshold": "multipartUploadThreshold", "multipartUploadSize": "multipartUploadSize", "s3Options": { "accessKeyId": "s3accessKey","signature":"calculatedSignature"}, "params": {"s3Params": { "Bucket": "S3bucketName", "Key": "key"}}}
```

###>> Socket Name: uploadmediasuccess

This event listener gives us the end uploaded URL, once upload is successful.

Sample Input Data:

```
{"dlink":{"username":"username","roomid":"roomid","filetype":"filetype","downloadlink":"downloadlink","timestamp":"time"}}
```

###>> Socket Name: useraccepted

When a user has accepted a link request for a chat room, which has been sent to the user in the form of SMS or Email, the client emits this event in order to update the server of the user's acceptance.
Sample Input Data:

```
{"roomid":"roomid","userid":"userid"}
```

###>> Socket Name: newuseradded

Upon successful user acceptance(through useraccepted socket), the server emits this event to all other user's in the room.
Sample Input Data:

```
{"userid":"userid", "roomid":"roomid"}
```

###>> Socket Name: userremoved

It is emitted when a user leaves a chat room or when a admin removes a user from a chat room. A client emits this event to the server and the server intimates this to other users in the room.
Sample Input Data:

```
{"username":"username","userid":"userid","roomid":"roomid","timestamp":"timeStamp"}
```

###>> Socket Name: useradded

It is emitted when a user is added to a chat room or when a admin adds a user to a chat room. A client emits this event to the server and the server intimates this to other users in the room.
Sample Input Data:

```
{"userid":"userid", "roomid":"roomid"}
```

###>> Socket Name: newroomadded

It is emitted when a new room is added to the chat list of a user.
Sample Input Data:

```
{"roomid":"roomid",userid[]}
```
###>> Socket Name: chatinprogress

It updates the server with the info that the particular chat room is active in the userid(client).So, the message unread count can be reset. To be called when ever the particular chat room is opened(progress=true), and while exit form a chat room(progress=false)
Sample Input Data:

```
{"roomid":"roomid","userid":"userid","progress":"progress"}
```
