module.exports = function(io, logger) {
    var db = require('../models/schema');
    var config = require('../config/config');
    var room = require('./room.js');
    var notify = require("push-notify");
    var crypto = require('crypto');
    var util = require('../util/util');
    const ANNOUNCEMENT = "announcement";
    const NEWMESSAGE = "newmessage";

    var usernames = {};
    // rooms which are currently available in chat
    var rooms = [];
    //An utility for getting current timestamp

    io.sockets.on('connection', function(socket) {
        // when the client emits 'adduser', this listens and executes
        socket.on('adduser', function(data) {
            logger.info('RoomID:' + data.roomid + 'UserName:' + data.userid + 'adduser :' + data.userid + 'to a new room :' + data.roomid + 'request');
            var timeStamp = util.getCurrentTimestamp();
            var name;
            var JSONobj = new Object();
            // store the username in the socket session for this client
            socket.username = data.userid;
            // store the room name in the socket session for this client
            socket.room = data.roomid;
            // add the client's username to the global list
            usernames[name] = data.userid;

            // send client to socket.room
            socket.join(data.roomid);

            // echo to client they've connected
            JSONobj = {
                "roomid": socket.room
            };
            // echo to socket.room that a person has connected to their room
            JSONobj = {
                "userid": data.userid,
                "roomid": data.roomid,
                "timestamp": timeStamp
            };

            try {
                var room = io.sockets.adapter.rooms[socket.room];
                var numberOfUsers = Object.keys(room).length;
                console.log("ROOMS Number of participants:" + numberOfUsers);
            } catch (err) {
                console.log("ROOMS error getting the participants");
            }
            util.auditlog(data.userid, 'user added :' + data.roomid);
        });


        makeMediaUploadReady = function(jsonvalues, callback) {
            var time = util.getCurrentTimestamp();
            logger.info('RoomID:' + jsonvalues.roomid + 'userid:' + jsonvalues.userid + 'image upload request');
            var userId = jsonvalues.userid;
            var roomID = jsonvalues.roomid;
            var userName = jsonvalues.username;
            var fileName = jsonvalues.filename;
            var fileType = jsonvalues.filetype; //JPG,MOV
            var type = jsonvalues.type; //Video,image
            var keymain, keythumbnail, signatureMain, signatureThumbnail, policyBase64Main, policyBase64Thumbnail;
            keymain = roomID + '/' + time + fileName;
            keythumbnail = roomID + '/thumbnail/' + time + fileName;
            getSignature(config.S3bucketName, keymain, function(sig) {
                policyBase64Main = sig[0];
                signatureMain = sig[1];
            });
            getSignature(config.S3bucketName, keythumbnail, function(sig) {
                policyBase64Thumbnail = sig[0];
                signatureThumbnail = sig[1];
            });

            //Create an S3 bucket and send the details to client.
            var JSONs3details = new Object();
            JSONs3details = {
                'userid': userId,
                'username': userName,
                'roomid': roomID,
                'filetype': fileType,
                'type': type,
                'timestamp': time,
                'maxAsyncS3': config.maxAsyncS3, // this is the default
                's3RetryCount': config.s3RetryCount, // this is the default
                's3RetryDelay': config.s3RetryDelay, // this is the default
                'multipartUploadThreshold': config.multipartUploadThreshold, // this is the default (100 MB)
                'multipartUploadSize': config.multipartUploadSize, // this is the default (100 MB)

                's3Options': {
                    'accessKeyId': config.s3accessKey,
                },
                'params': {
                    'localFile': 'localFoldedPath+file',

                    's3Params': {
                        'bucket': config.S3bucketName,
                        'key': keymain,
                        'keyForThumbnail': keythumbnail,
                        'policy': policyBase64Main,
                        'policyForThumbnail': policyBase64Thumbnail,
                        'signature': signatureMain,
                        'signatureForThumbnail': signatureThumbnail
                    },
                }
            };
            callback(JSONs3details);
        }

        module.exports.makeMediaUploadReadyFromOutside = makeMediaUploadReady;


        updateRoom = function(attendees, roomID, lastmessage) {
            logger.info(":::ATTENDEES:" + attendees)
            db.chatGroup.update({
                roomId: roomID
            }, {
                $set: {
                    attendees: attendees,
                    lastChat: lastmessage
                }
            }, function(err, result) {
                if (err) {
                    logger.error("error in updating chatRoom");
                } else {
                    logger.info("Updated the ROOM");
                }
            });
        }

        saveLastMsgAndsendPush = function(newattendees, pushNotifyInfo) {
            logger.info("SaveLastMsgAndsendPush -User is a valid user :::" + pushNotifyInfo.attendee.unreadcount);
            var tempVales = {};
            if (pushNotifyInfo.senderuid == pushNotifyInfo.attendee.userid || pushNotifyInfo.attendee.chatprogress == true) {
                tempVales.unreadcount = 0;
            } else {
                tempVales.unreadcount = pushNotifyInfo.attendee.unreadcount + 1;
            }
            tempVales.userid = pushNotifyInfo.attendee.userid;
            tempVales.jointime = pushNotifyInfo.attendee.jointime;
            tempVales.contact = pushNotifyInfo.attendee.contact;
            tempVales.moc = pushNotifyInfo.attendee.moc;
            tempVales.username = pushNotifyInfo.attendee.username;
            tempVales.chatprogress = pushNotifyInfo.attendee.chatprogress;
            tempVales.initialurl = pushNotifyInfo.attendee.initialurl;
            tempVales.requestId = pushNotifyInfo.attendee.requestId;
            tempVales.lastremaindertime = pushNotifyInfo.attendee.lastremaindertime;

            logger.info("senderuid" + pushNotifyInfo.senderuid);
            logger.info("attendee userid" + pushNotifyInfo.attendee.userid);
            logger.info("attendee accepted status" + pushNotifyInfo.attendee.accepted);
            if ((pushNotifyInfo.senderuid == pushNotifyInfo.attendee.userid) && (!pushNotifyInfo.attendee.accepted)) {
                logger.info("condition satisfied");
                tempVales.accepted = true;
                updateAccceptance(pushNotifyInfo.roomID, pushNotifyInfo.room, tempVales);
            } else {
                tempVales.accepted = pushNotifyInfo.attendee.accepted;
            }

            if (pushNotifyInfo.attendee.profile_id == undefined) {
                logger.info("NOT sending push");
                newattendees[pushNotifyInfo.id] = tempVales;
                if (pushNotifyInfo.update)
                    updateRoom(newattendees, pushNotifyInfo.roomID, pushNotifyInfo.lastmessage);
            } else {
                logger.info("RR-Valid for push notification");
                tempVales.profile_id = pushNotifyInfo.attendee.profile_id._id;
                newattendees[pushNotifyInfo.id] = tempVales;
                logger.info("RR-Valid for push notification::" + pushNotifyInfo.attendee.chatprogress, pushNotifyInfo.senderuid, pushNotifyInfo.attendee.userid, pushNotifyInfo.attendee.profile_id.status);
                if (pushNotifyInfo.attendee.chatprogress != true && pushNotifyInfo.senderuid != pushNotifyInfo.attendee.userid && pushNotifyInfo.attendee.profile_id.status != "loggedout" && pushNotifyInfo.attendee.profile_id.status != "offline") {
                    logger.info("sending push");
                    db.chatGroup.aggregate(
                        [{
                            $match: {
                                'attendees': {
                                    $elemMatch: {
                                        'userid': pushNotifyInfo.attendee.profile_id.userId
                                    }
                                }
                            }
                        }, {
                            $unwind: "$attendees"
                        }, {
                            $match: {
                                'attendees.userid': pushNotifyInfo.attendee.profile_id.userId
                            }
                        }, {
                            $group: {
                                _id: null,
                                count: {
                                    $sum: '$attendees.unreadcount'
                                }
                            }
                        }],
                        function(err, outcome) {
                            logger.info("Total unread: " + JSON.stringify(outcome));
                            var totalUnread = 0;
                            if (outcome.length != 0 && outcome[0].count != "undefined") {
                                totalUnread = outcome[0].count + 1;
                            }
                            var dataToSend = {
                                fromuserid: pushNotifyInfo.lastmessage.userId,
                                touserid: pushNotifyInfo.attendee.userid,
                                chattype: pushNotifyInfo.lastmessage.chatType,
                                message: pushNotifyInfo.lastmessage.messsage,
                                timestamp: pushNotifyInfo.lastmessage.timestamp,
                                roomid: pushNotifyInfo.roomID,
                                roomunread: tempVales.unreadcount,
                                allunread: totalUnread
                            }
                            if (pushNotifyInfo.update) {
                                logger.info("Room update called");
                                updateRoom(newattendees, pushNotifyInfo.roomID, pushNotifyInfo.lastmessage);
                            }
                            if (pushNotifyInfo.notify)
                                util.pushNotify(pushNotifyInfo.attendee.profile_id.deviceInfo[0].devicetype, pushNotifyInfo.attendee.profile_id.deviceInfo[0].devicetoken, pushNotifyInfo.roomID, pushNotifyInfo.attendee.userid, pushNotifyInfo.memberscount, NEWMESSAGE, dataToSend);
                        });
                } else {
                    if (pushNotifyInfo.update)
                        updateRoom(newattendees, pushNotifyInfo.roomID, pushNotifyInfo.lastmessage);
                }

            }
        }



        updateLastChatAndPushNotify = function(roomID, lastmessage, senderuid, isAdminRoom, adminRoom, notify) {
            db.chatGroup.findOne({
                roomId: roomID
            }).populate("attendees.profile_id").exec(function(err, room) {
                if (err) {
                    logger.info("error in finding chatRoom");
                } else if (room == null || room == undefined) {
                    logger.info("Create Admin room for :" + roomID);
                    //Create a new room
                    var userIdOfAttendee = roomID;
                    if (isAdminRoom == true && roomID != config.adminUserId) {
                        room.createNewSelfRoom(userIdOfAttendee, lastmessage, adminRoom, notify);
                    }
                } else {
                    var copyOfRoom;
                    copyOfRoom = room;
                    var attendeescount = copyOfRoom.attendees.length;
                    var newattendees = [];
                    for (var i = 0; i < attendeescount; i++) {
                        if (i == attendeescount - 1) {
                            var pushNotifyInfo = {
                                room: copyOfRoom,
                                attendee: copyOfRoom.attendees[i],
                                update: true,
                                newattendees: newattendees,
                                id: i,
                                roomID: roomID,
                                lastmessage: lastmessage,
                                senderuid: senderuid,
                                memberscount: attendeescount,
                                notify: notify
                            };
                            saveLastMsgAndsendPush(newattendees, pushNotifyInfo);
                        } else {
                            var pushNotifyInfo = {
                                room: copyOfRoom,
                                attendee: copyOfRoom.attendees[i],
                                update: false,
                                newattendees: newattendees,
                                id: i,
                                roomID: roomID,
                                lastmessage: lastmessage,
                                senderuid: senderuid,
                                memberscount: attendeescount,
                                notify: notify
                            };
                            saveLastMsgAndsendPush(newattendees, pushNotifyInfo);
                        }
                    }
                }
            });
        }

        module.exports.updateLastChatAndPush = updateLastChatAndPushNotify;

        updateChatInPrgress = function(roomID, userid, isProgress) {
            var timeStamp = util.getCurrentTimestamp();
            logger.info('updateChatInPrgress update:' + roomID + "to " + isProgress);
            if (roomID != "") {
                db.chatGroup.update({
                    $and: [{
                        'roomId': roomID
                    }, {
                        "attendees": {
                            $elemMatch: {
                                "userid": userid,
                                "jointime": ""
                            }
                        }
                    }]
                }, {
                    $set: {
                        'attendees.$.chatprogress': isProgress,
                        'attendees.$.unreadcount': 0,
                        'attendees.$.jointime': timeStamp
                    }
                }, function(err, updatedGroup) {
                    if (err) {
                        logger.error('updateChatInPrgress error:' + err);
                        logger.info('updateChatInPrgress error:1');
                    } else {
                        if (updatedGroup.nModified) {
                            logger.info('User joined chat room: updated ChatInPrgress,jointime' + JSON.stringify(updatedGroup));
                        } else {
                            db.chatGroup.update({
                                $and: [{
                                    'roomId': roomID
                                }, {
                                    "attendees": {
                                        $elemMatch: {
                                            "userid": userid
                                        }
                                    }
                                }]
                            }, {
                                $set: {
                                    'attendees.$.chatprogress': isProgress,
                                    'attendees.$.unreadcount': 0
                                }
                            }, function(err, updatedGroupResult) {
                                if (err) {
                                    logger.error('updateChatInPrgress error:' + err);
                                    logger.info('updateChatInPrgress error:2');
                                } else {
                                    logger.info('updated ChatInPrgress');
                                }

                            });
                        }
                    }

                });
            } else {
                db.chatGroup.update({
                    "attendees": {
                        $elemMatch: {
                            "userid": userid
                        }
                    }
                }, {
                    $set: {
                        'attendees.$.chatprogress': false
                    }
                }, {
                    multi: true
                }, function(err, updatedGroup) {
                    if (err) {
                        logger.error('updateChatInPrgress error:' + err);
                    } else {
                        logger.info('updated ChatInPrgress');
                    }

                });
            }
        }


        // when the client emits 'sendchat', this listens and executes
        socket.on('sendchat', function(data) {
            logger.info("data" + JSON.stringify(data));
            logger.info('RoomID:' + data.roomid + 'UserName:' + data.userid + 'sendChat');
            var time = util.getCurrentTimestamp();
            var message = data.message;
            var userId = data.userid;
            var chatType = data.type;
            var roomID = data.roomid;
            var userName = data.username;
            var numberOfUsers;
            logger.info("received:" + JSON.stringify(data));
            // we tell the client to execute 'updatechat' with 2 parameters
            var JSONchat = new Object();
            JSONchat = {
                    "userid": userId,
                    "username": userName,
                    "roomid": roomID,
                    "type": chatType,
                    "message": data.message,
                    "timestamp": time
                }
                //Find if the roomid and userid same or
                //roomid is Admin rooms
                //if userid is userid of admin
                //Save an entry in the History of chat
            var commLog = new db.chatlog({
                roomId: roomID,
                from: userId,
                username: userName,
                userId: userId,
                type: chatType,
                messsage: message,
                timestamp: time
            });
            commLog.save(function(err, userObj) {
                if (err) {
                    logger.error(err);
                } else {
                    logger.info('saved communication log successfully:');
                }
            });

            if (userId == roomID) {
                //Admin room
                logger.info("AdminRoom- Response not processed:" + roomID);
            } else {
                socket.broadcast.to(roomID).emit('updatechat', JSONchat);
                socket.emit('updatechat', JSONchat);
                try {
                    var room = io.sockets.adapter.rooms[socket.room];
                    numberOfUsers = Object.keys(room).length;
                    console.log("ROOMS Number of participants:" + numberOfUsers);
                } catch (err) {
                    console.log("ROOMS error getting the participants");
                }
                //Update the latest chat in chatGroup
                updateLastChatAndPushNotify(roomID, {
                    userId: userId,
                    chatType: chatType,
                    messsage: message,
                    timestamp: time
                }, userId, false, "", true);
            }
        });

        socket.on('adminnotificaion', function(JSONObj, userInfo) {
            if (userInfo.type != 'text') {
                socket.broadcast.to(userInfo.roomid).emit('uploadmediasuccess', JSONObj);
            } else {
                socket.broadcast.to(userInfo.roomid).emit('updatechat', JSONObj);
            }
            var commLog = new db.chatlog({
                roomId: userInfo.roomid,
                from: userInfo.userid,
                username: userInfo.username,
                userId: userInfo.userid,
                type: userInfo.type,
                messsage: userInfo.message,
                timestamp: userInfo.time
            });
            commLog.save(function(err, userObj) {
                if (err) {
                    logger.error(err);
                } else {
                    logger.info('saved communication log successfully:');
                }
            });
        });


        // When an user tries to type(keyboard active) it will be intimated to all the clients connected in the room
        socket.on('typeinprogress', function(data) {
            logger.info('RoomID:' + data.roomid + 'UserName:' + data.userid + 'typeinprogress');
            var JSONobj = new Object();
            //send username and the room info that he is typing
            JSONobj = {
                "userid": data.userid,
                "username": data.username,
                "roomid": data.roomid
            };
            socket.broadcast.to(data.roomid).emit('typeinprogress', JSONobj);
        });

        // When an user stops typing(keyboard inactive) it will be intimated to all the clients connected in the room
        socket.on('typeinstopped', function(data) {
            logger.info('RoomID:' + data.roomid + 'userId:' + data.userid + 'typeinstopped');
            var JSONobj = new Object();
            //send username and the room info that he is typing
            JSONobj = {
                "userid": data.userid,
                "username": data.username,
                "roomid": data.roomid
            };
            socket.broadcast.to(data.roomid).emit('typeinstopped', JSONobj);
        });



        socket.on('userremoved', function(data) {
            var timeStamp = util.getCurrentTimestamp();
            logger.info('username:' + data.username + 'RoomID:' + data.roomid + 'userId:' + data.userid + 'userremoved');
            //logger.info("timestamp:" +time);
            var JSONobj = new Object();
            //send userremoved to update the clients
            JSONobj = {
                "username": data.username,
                "userid": data.userid,
                "roomid": data.roomid,
                "timestamp": timeStamp
            };
            logger.info("jsonvalues:" + JSON.stringify(JSONobj));
            socket.broadcast.to(data.roomid).emit('userremoved', JSONobj);
            util.auditlog(data.userid, 'userremoved from room :' + data.roomid);
            var commLog = new db.chatlog({
                roomId: data.roomid,
                from: data.userid,
                username: data.username,
                userId: data.userid,
                type: "notification",
                messsage: data.username + " left the chat",
                timestamp: timeStamp
            });
            commLog.save(function(err, userObj) {
                if (err) {
                    logger.error(err);
                } else {
                    logger.info('saved communication log successfully:');
                }
            });
        });

        socket.on('useradded', function(data) {
            var time = util.getCurrentTimestamp();
            logger.info('RoomID:' + data.roomid + 'userId:' + data.userid + 'useradded');
            var JSONobj = new Object();
            //send userremoved to update the clients
            JSONobj = {
                "userid": data.userid,
                "roomid": data.roomid
            };
            socket.broadcast.to(data.roomid).emit('useradded', JSONobj);
        });

        socket.on('newroomadded', function(data) {
            var time = util.getCurrentTimestamp();
            logger.info('RoomID:' + data.roomid + 'newroomadded');
            socket.broadcast.emit('newroomadded', data);
        });

        socket.on('chatinprogress', function(data) {
            var time = util.getCurrentTimestamp();
            logger.info('RoomID:' + data.roomid + 'userid' + data.userid + "progress" + data.progress);
            updateChatInPrgress(data.roomid, data.userid, data.progress);
        });


        socket.on('useraccepted', function(data) {
            var time = util.getCurrentTimestamp();
            logger.info('RoomID:' + data.roomid + 'userId:' + data.userid + 'useraccepted');
            var JSONobj = new Object();
            //send userremoved to update the clients
            JSONobj = {
                "userid": data.userid,
                "roomid": data.roomid
            };
            if (data.roomid == "") {
                socket.broadcast.emit('newuseradded', JSONobj);
            } else {
                socket.broadcast.to(data.roomid).emit('newuseradded', JSONobj);
            }
        });

        getSignature = function(bucket, key, callback) {
            var policy, policyBase64, signature;
            var result = [];

            var expiry = new Date(new Date().getTime() + 10000 * 60 * 15).toISOString();
            var policy = {
                "expiration": expiry,
                "conditions": [{
                        "bucket": bucket
                    }, {
                        "key": key
                    }, {
                        "acl": 'public-read'
                    },
                    ["starts-with", "$Content-Type", ""],
                    ["content-length-range", 0, 524288000]
                ]
            };

            result[0] = new Buffer(JSON.stringify(policy), 'utf8').toString('base64');
            result[1] = crypto.createHmac('sha1', config.s3secretKey).update(result[0]).digest('base64');
            console.log('policyBase64:' + result[0]);
            console.log('signature is:' + result[1]);

            return callback(result);

        }

        //When a client sends a request to upload media in s3bucket, give the URL and details of s3 bucket to client.
        //Clients can use link to upload the media to s3
        socket.on('uploadmedia', function(data) {
            var jsondata = {
                userId: data.userid,
                roomID: data.roomid,
                userName: data.username,
                fileName: data.filename,
                fileType: data.filetype, //JPG,MOV
                type: data.type //Video,image
            };
            makeMediaUploadReady(jsondata, function(dataToSend) {
                socket.emit('uploadmedia', dataToSend);
                util.auditlog(jsondata.userId, 'uploadmedia requested :' + jsondata.roomID);
            });
        });


        //Once the media upload is success client will intimate the media upload success
        //Once receiving the success info, send the link and details to every other clients for downloading
        socket.on('uploadmediasuccess', function(data) {
            logger.info('RoomID:' + data.roomid + 'userId:' + data.userid + 'uploadmediasuccess request');
            var time = util.getCurrentTimestamp();
            var userId = data.userid;
            var userName = data.username;
            var roomID = data.roomid;
            var fileType = data.filetype //JPG,MOV
            var type = data.type; //Video,image
            var key = data.key;
            var keyForThumbnail = data.keyforthumbnail;
            logger.info("media file name :" + key);
            var thumbdlink = config.downloadlink + '/' + keyForThumbnail;
            //Populate the S3 bucket download link to other client.
            var JSONdlinkdetails = new Object();
            JSONdlinkdetails = {

                'dlink': {
                    'userid': userId,
                    'username': userName,
                    'roomid': roomID,
                    'filetype': fileType,
                    'type': type,
                    'downloadlink': config.downloadlink + '/' + key,
                    'downloadlinkforthumbnail': config.downloadlink + '/' + keyForThumbnail,
                    'timestamp': time
                }

            };

            socket.broadcast.to(roomID).emit('uploadmediasuccess', JSONdlinkdetails);
            socket.emit('uploadmediasuccess', JSONdlinkdetails);
            //Save an entry in the chatLog
            var commLog = new db.chatlog({
                roomId: roomID,
                username: userName,
                from: userId,
                type: type,
                userId: userId,
                messsage: config.downloadlink + '/' + key + ';' + thumbdlink,
                timestamp: time
            });
            commLog.save(function(err, userObj) {
                if (err) {
                    logger.error(err);
                } else {
                    logger.info('saved communication log successfully:');
                }
            });

            //Update the latest chat in chatGroup
            updateLastChatAndPushNotify(roomID, {
                userId: userId,
                chatType: type,
                messsage: config.downloadlink + '/' + key + ';' + thumbdlink,
                timestamp: time
            }, userId, false, "", true);

        });


        // when the user disconnects.. perform this
        socket.on('disconnect', function() {
            var time = util.getCurrentTimestamp();
            console.log("Socket disconnected for user:" + socket.username + "from room:" + socket.room);
            // remove the username from global usernames list
            delete usernames[socket.username];
            // update list of users in chat, client-side
            io.sockets.emit('updateusers', usernames);
            // echo globally that this client has left
            var JSONdisconnectinfo = new Object();
            JSONdisconnectinfo = {
                "userid": socket.username,
                "roomid": socket.room
            };
            socket.leave(socket.room);
            socket.broadcast.emit('leftroom', JSONdisconnectinfo);
            if (socket.username != undefined && socket.room != undefined) {
                updateChatInPrgress(socket.room, socket.username, false);
            }
        });
    });
}
