var uuid = require('node-uuid')
var db = require('../models/schema')
var config = require('../config/config')
var util = require('../util/util')
var log = require('../lib/logger.js')
var emailService = require('../emailservice/emailservice')
var smsService = require('../smsservice/sms')
var emptyString = ''
const ANNOUNCEMENT = "announcement";

// Find the attendees list for the group
module.exports.findAttendeesListForAGroup = function(roomid, callback) {
    try {
        log.info('findAttendeesListForAGroup :' + roomid)
        var attendees = []
        findGroupForTheRoomId(roomid, function(response) {
            if (!response || !Array.isArray(response) || response.length === 0) {
                return callback(emptyString)
            } else {
                log.info('findGroupForTheRoomId result attendees: ' + response.attendees)
                attendees = response.attendees
                return callback(attendees)
            }
        })
    } catch (err) { // eslint-disable-line no-tabs
        log.error('error in findAttendeesListForAGroup' + err)
    }
}

// validate for creating room
validateForRoomCreation = function(jsondata, callback) {
    log.info("validate for room creation");
    if (!jsondata.userid) {
        log.error('error createChat: notavalidinput')
        callback(417, {
            'error': 'notavalidinput'
        })
    }
    // Find for existing room
    if (jsondata.userid && jsondata.targetUserId) {
        findForExistingRoom(jsondata, function(code, result) {
            callback(code, result)
        })
    } else if (!jsondata.groupname || jsondata.groupname.length === 0) {
        callback(417, {
            'error': 'noroomname'
        })
    } else {
        // create new room
        createNewRoom(jsondata, function(code, result) {
            log.info('createNewRoom result:' + JSON.stringify(result))
            callback(code, result)
        })
    }
}

// Create a chat room
// Params: userid,groupname,isgroupchatflag
module.exports.createChat = function(req, res) {
    try {
        log.info('createChat req params:' + req.body.userid.trim() + ',' + req.body.groupname + ',' + req.body.isgroupchat + ',' + req.body.targetuserid)
        var userId = req.body.userid.trim()
        var groupName = req.body.groupname
        var userDetails = req.body.userdetails // {username,contact,moc}
        var summary = req.body.summary
        var time = util.getCurrentTimestamp()
        var targetUserId = req.body.targetuserid
        var clientTime = req.body.timestamp
        var purpose = req.body.purpose || ''
        var roomId = uuid.v1()
        var jsondata = {
            'userid': userId,
            'groupname': groupName,
            'summary': summary,
            'userdetails': userDetails,
            'time': time,
            'purpose': purpose,
            'clienttime': clientTime,
            'roomId': roomId,
            'targetUserId': targetUserId,
            'chatprogress': true
        }
        validateForRoomCreation(jsondata, function(code, result) {
            jsondata.roomid = result.roomid;
            res.status(code).json(result)
        })
    } catch (err) {
        log.info('error in createChat' + err)
        res.status(500).json({
            'error': 'error in creating chat room'
        })
    }
}

findForExistingRoom = function(jsondata, callback) {
    db.chatGroup.findOne({
        $and: [{
            'attendeesBackUp': {
                $size: 2
            }
        }, {
            $and: [{
                'attendeesBackUp': {
                    $elemMatch: {
                        'userid': jsondata.userid
                    }
                }
            }, {
                'attendeesBackUp': {
                    $elemMatch: {
                        'userid': jsondata.targetUserId.trim()
                    }
                }
            }, {
                'purpose': 'chat'
            }]
        }]
    }, function(err, existingroom) {
        if (err) {
            log.error('Error in finding Private chat userid ' + userId + targetUserId)
            callback(417, {
                'error': '"Error in finding Private chat userid'
            })
        } else if (existingroom == null) {
            log.info('No result found for privatechat')
            createNewRoom(jsondata, function(code, result) {
                log.info('createNewRoom result:' + JSON.stringify(result))
                callback(code, result)
            })
        } else {
            findProfileAndUpdate(jsondata, existingroom, function(code, result) {
                log.info('createNewRoom result:' + JSON.stringify(result))
                callback(code, result)
            })
        }
    })
}
findProfileAndUpdate = function(jsondata, existingroom, callback) {
    db.profiles.findOne({
        'userId': jsondata.userid
    }, function(err, profile) {
        if (err) {
            log.info('chatComm:createChat:ERROR in finding profile')
            callback(500, {
                'error': 'error in creating chat room'
            })
        } else if (profile) {
            var oldroom = existingroom.roomId
            log.info('createchat existing room found :' + oldroom)
            if (jsondata.summary != '') {
                updateChatGroupWithSummary(oldroom, jsondata, profile)
                jsondata.roomId = oldroom
                saveSummaryInChatLog(jsondata)
            } else {
                updateChatGroup(oldroom, jsondata, profile)
            }
            callback(200, {
                'roomid': oldroom
            })
        } else {
            log.info('chatComm:createChat:no profile found')
            callback(417, {
                'error': 'Not a registered user'
            })
        }
    })
}
updateChatGroupWithSummary = function(oldroom, jsondata, profile) {
    db.chatGroup.update({
        'roomId': oldroom
    }, {
        admins: [jsondata.userid, jsondata.targetUserId.trim()],
        lastremaindertime: '',
        message: jsondata.summary,
        timestamp: jsondata.time,
        attendees: [{
            'userid': jsondata.userid,
            'jointime': jsondata.time,
            'lastremaindertime': '',
            'contact': jsondata.userdetails.contact,
            'moc': jsondata.userdetails.moc,
            'username': jsondata.userdetails.username,
            'unreadcount': 1,
            'chatprogress': jsondata.chatprogress,
            'accepted': false,
            profile_id: profile._id
        }],
        attendeesBackUp: [{
            'userid': jsondata.userid,
            'jointime': jsondata.time,
            'lastremaindertime': '',
            'contact': jsondata.userdetails.contact,
            'moc': jsondata.userdetails.moc,
            'username': jsondata.userdetails.username,
            'unreadcount': 1,
            'chatprogress': jsondata.chatprogress,
            'accepted': false,
            profile_id: profile._id
        }],
        lastChat: {
            userId: jsondata.userid,
            chatType: 'text',
            messsage: jsondata.summary,
            timestamp: jsondata.time
        }
    }, {
        multi: true
    }, function(err, deletedresult) {
        if (err) {
            log.error('error:', err)
        } else {
            if (!deletedresult.nModified) {
                log.info('createchat result not modified')
            } else {
                log.info('createchat result:' + JSON.stringify(deletedresult))
            }
        }
    })
}
updateChatGroup = function(oldroom, jsondata, profile) {
    db.chatGroup.update({
        'roomId': oldroom
    }, {
        admins: [jsondata.userid, jsondata.targetUserId.trim()],
        lastremaindertime: '',
        timestamp: jsondata.time,
        attendees: [{
            'userid': jsondata.userid,
            'jointime': jsondata.time,
            'lastremaindertime': '',
            'contact': jsondata.userdetails.contact,
            'moc': jsondata.userdetails.moc,
            'username': jsondata.userdetails.username,
            'unreadcount': 0,
            'chatprogress': jsondata.chatprogress,
            'accepted': false,
            profile_id: profile._id
        }],
        attendeesBackUp: [{
            'userid': jsondata.userid,
            'jointime': jsondata.time,
            'lastremaindertime': '',
            'contact': jsondata.userdetails.contact,
            'moc': jsondata.userdetails.moc,
            'username': jsondata.userdetails.username,
            'unreadcount': 0,
            'chatprogress': jsondata.chatprogress,
            'accepted': false,
            profile_id: profile._id
        }]
    }, {
        multi: true
    }, function(err, deletedresult) {
        if (err) {
            log.error('error:', err)
        } else {
            if (!deletedresult.nModified) {
                log.info('createchat result not modified')
            } else {
                log.info('createchat result:' + JSON.stringify(deletedresult))
            }
        }
    })
}


createNewRoom = function(jsondata, callback) {
    db.profiles.findOne({
        'userId': jsondata.userid
    }, function(err, profile) {
        if (err) {
            log.info('chatComm:createChat:ERROR in finding profile')
            callback(500, {
                'error': 'error in creating chat room'
            })
        } else {
            if (profile) {
                var group = new db.chatGroup({
                    groupName: jsondata.groupname,
                    roomId: jsondata.roomId,
                    attendees: {
                        'userid': jsondata.userid,
                        'requestId': '',
                        'jointime': jsondata.time,
                        'lastremaindertime': '',
                        'initialurl': '',
                        'contact': jsondata.userdetails.contact,
                        'moc': jsondata.userdetails.moc,
                        'username': jsondata.userdetails.username,
                        'unreadcount': 0,
                        'chatprogress': jsondata.chatprogress,
                        'accepted': true,
                        'profile_id': profile._id
                    },
                    attendeesBackUp: {
                        'userid': jsondata.userid,
                        'requestId': '',
                        'jointime': jsondata.time,
                        'lastremaindertime': '',
                        'initialurl': '',
                        'contact': jsondata.userdetails.contact,
                        'moc': jsondata.userdetails.moc,
                        'username': jsondata.userdetails.username,
                        'unreadcount': 0,
                        'chatprogress': jsondata.chatprogress,
                        'accepted': true,
                        'profile_id': profile._id
                    },
                    admins: [jsondata.userid],
                    lastremaindertime: '',
                    message: jsondata.summary,
                    timestamp: jsondata.time,
                    purpose: jsondata.purpose,
                    clienttime: jsondata.time,
                    lastChat: {
                        userId: jsondata.userid,
                        chatType: 'text',
                        messsage: jsondata.summary,
                        timestamp: jsondata.time
                    }
                })
                group.save(function(err, userObj) {
                    if (err) {
                        log.error(err)
                    } else {
                        log.info('saved initiator details successfully:')
                        saveSummaryInChatLog(jsondata)
                        callback(200, {
                            'roomid': jsondata.roomId
                        })
                    }
                })
            } else {
                log.info('chatComm:createChat:no profile found')
                callback(417, {
                    'error': 'Not a registered user'
                })
            }
        }
    })
}
module.exports.createNewRoom = createNewRoom;

saveSummaryInChatLog = function(jsondata) {
    if (jsondata.summary != '') {
        // Save an entry in the History of chat
        var commLog = new db.chatlog({
            roomId: jsondata.roomId,
            from: jsondata.userid,
            username: jsondata.userdetails.username,
            userId: jsondata.userid,
            type: 'text',
            messsage: jsondata.summary,
            timestamp: jsondata.time
        })
        commLog.save(function(err, userObj) {
            if (err) {
                log.error(err)
            } else {
                log.info('saved communication log successfully:')
            }
        })
    }
}

// Invite members to the group chat
// params: userid,roomid,pushnotify,isgroupchatflag
module.exports.inviteMembers = function(req, res) {
    try {
        log.info(' inviteMembers req :' + req.body.roomid)

        var userId = req.body.userid.trim()
        var notifyMail = req.body.pushnotify
        var roomId = req.body.roomid.trim()
        var groupName = req.body.groupname.trim()
        var memberCount
        var Members = []
        var responseToSend = []

        Members = req.body.members

        if (typeof Members !== 'undefined' && Members != null) {
            memberCount = Members.length + 1
            addMemberAndNotify(Members, roomId, userId, memberCount, notifyMail, groupName)
        }
        log.info('membersCount', memberCount)
            // Check if it is group chat
            // Invite members
            // add members to the group

        if (!roomId || !userId) {
            lor.error('error : inviteMembers notavalidinput')
            res.status(417).json({
                'error': 'notavalidinput'
            })
            return
        }

        res.json({
            'result': responseToSend
        })
    } catch (err) {
        log.error('error in inviteMembers:' + err)
        res.status(500).json({
            'error': 'error in inviteMembers'
        })
    }
}

addMemberAndNotify = function(members, roomId, userId, memberCount, notifyMail, groupName) {
    try {
        var responseToSend = []
        var invitedthrough = 'chat'
        log.info('addMemberAndNotify')
        var remindedthrough = 'inviteMembers';
        var time = util.getCurrentTimestamp()
        var requestId = uuid.v1() + ":" + time;
        // Ignore auto reminder execution in case of manual reminders
        util.getPurposeFromRoomId(roomId, function(result) {
            if (result.purpose != 'manualreminder') {
                log.info("addMemberAndNotify1 - autoReminder scheduled");
            }
        })
        var newroomdata = {};
        newroomdata.roomid = roomId;
        newroomdata.userid = [];
        for (i in members) {
            if (members.length > 1) {
                members[i].emailtemplate = 'groupchat'
                members[i].memberscount = memberCount
            } else {
                members[i].emailtemplate = 'chat'
                members[i].memberscount = memberCount
            }
            if (members[i].userid && members[i].userid.length !== 0) {
                newroomdata.userid.push(members[i].userid);
            }
            addMemberToGroup(userId, roomId, groupName, members[i], notifyMail, requestId, "", function(result) {
                if (!result) {
                    responseToSend[i] = {
                        'error': 'notthere'
                    }
                } else {
                    responseToSend[i] = result
                }
            })
        }
        app.locals.socketClient.emit('newroomadded', newroomdata);
    } catch (err) {
        log.error('error in addMemberAndNotify:' + err)
    }
}


// Find the Attendees list for an existing group
// params: userid,roomid
module.exports.attendeesList = function(req, res) {
    try {
        log.info('attendeesList req api :' + req.body.roomid)
        var attendees
        var admin
        var remainderSentTime
        var roomId = req.body.roomid.trim()
        var userId = req.body.userid.trim()
        if (!roomId || !userId) {
            log.error('error in attendeesList: notavalidinput')
            res.status(417).json({
                'error': 'notavalidinput'
            })
            return
        }
        // fetch the attendees list for the group
        findGroupForTheRoomId(roomId, function(response) {
            if (!response) {
                log.error('attendeesList not found')
                res.status(417).json({
                    'attendees': 'notthere'
                })
            } else {
                log.info('findGroupForTheRoomId result attendees: ' + response.attendees + '::' + response.admin + '::' + response.lastremaindertime)
                admin = response.admins
                remainderSentTime = response.lastremaindertime
                var purpose = response.purpose
                res.json({
                    'attendees': response.attendees,
                    'admin': admin,
                    'lastremaindertime': remainderSentTime,
                    'purpose': purpose
                })
            }
        })
    } catch (err) {
        log.error('error in attendeesList:' + err)
        res.status(500).json({
            'error': 'error in attendeesList'
        })
    }
}


updateChatGroupWithChatUrl = function(params) {
    db.chatGroup.update({
        $and: [{
            'roomId': params.roomId
        }, {
            'attendees.contact': params.member.contact
        }]
    }, {
        $set: {
            'attendees.$.initialurl': params.status.createdlink
        }
    }, function(err, result) {
        if (err) {
            log.info('error:', err)
        } else {
            db.chatGroup.update({
                $and: [{
                    'roomId': params.roomId
                }, {
                    'attendeesBackUp.contact': params.member.contact
                }]
            }, {
                $set: {
                    'attendeesBackUp.$.initialurl': params.status.createdlink
                }
            }, function(err, result) {
                if (err) {
                    log.info('error:', error)
                } else {
                    log.info('result:', result)
                }
            })
        }
    })
}

updateAndMailToMember = function(emailData, query, member, notifyMail, callback) {
    db.chatGroup.update({
        roomId: query.roomId,
        "attendees": {
            "$not": {
                "$elemMatch": {
                    "contact": query.contact
                }
            }
        }
    }, {
        '$addToSet': {
            attendees: query,
            attendeesBackUp: query
        },
        'groupName': query.groupName /*,'$addToSet':{'requests':query.requestId}*/
    }, function(err, result) {
        if (err) {
            log.info('error:', err)
        } else {
            db.chatGroup.update({
                    roomId: query.roomId
                }, {
                    '$addToSet': {
                        'requests': query.requestId
                    }
                }, function(err, result) {
                    if (err) {
                        log.info('error:', error)
                    } else {
                        log.info('result:', result)
                    }
                })
                // Following code block will  execute only for chat invite
            if (notifyMail) {
                if (member.moc == 'sms') {
                    log.info('MOC is SMS:' + member.moc)
                    smsService.sendSMS(member.contact, emailData, function(resultOfSMS) {
                        smsSendStatus = resultOfSMS
                        var urlData = {
                            'roomId': query.roomId,
                            'member': member,
                            'status': smsSendStatus
                        }
                        updateChatGroupWithChatUrl(urlData)
                        log.info('SMS send status for added member:' + JSON.stringify(smsSendStatus))
                    })
                } else if (member.moc == 'email') {
                    log.info('MOC is email:' + member.moc)
                    emailService.mailSender(member.username, member.contact, 'token', query.roomId, emailData, function(result) {
                        mailSendStatus = result
                        var urlData = {
                            'roomId': query.roomId,
                            'member': member,
                            'status': mailSendStatus
                        }
                        updateChatGroupWithChatUrl(urlData)
                        log.info('Mail send status for added member:' + JSON.stringify(mailSendStatus))
                    })
                } else {
                    log.info('Invalid MOC:' + member.moc)
                }
            }
            log.info(result)
            if (!result || !Array.isArray(result) || result.length === 0 || !result.nModified) {
                callback({
                    'error': 'failedtoadd'
                })
            } else {
                callback({
                    'member': member
                })
            }
        }
    }, {
        multi: true
    })
}

// Add a member to group
addMemberToGroup = function(userId, roomid, groupName, member, notifyMail, requestId, requestType, callback) {
    try {
        log.info('adding member ' + JSON.stringify(member) + 'to roomId ' + roomid)

        var mailSendStatus
        var smsSendStatus
        var usernameOfInvitingPerson
        // Check if the userId is an Admin
        validateForAdminUser(userId, roomid, function(isAdmin, groupInfo) {
            log.info(userId + ' addMemberToGroup:is a valid admin ' + isAdmin)
            if (isAdmin) {
                var clientTime = new Date(groupInfo.clienttime)
                groupName = groupInfo.groupName
                db.profiles.findOne({
                    'userId': userId
                }, function(err, userResult) {
                    if (err) {
                        log.error('addMemberToGroup:Error in getprofile method for userId ' + userId + ' ' + err)
                    } else {
                        if (userResult === null) {
                            log.info('addMemberToGroup:No Matching records found for userId ' + userId)
                        } else {
                            log.info('addMemberToGroup:Record retrieved successfully for userid ' + userId)
                            usernameOfInvitingPerson = userResult.username
                                // Check if the member is a chat member and add the profileID in attendees
                            var queryToFind
                            if (member.moc === 'sms') {
                                queryToFind = {
                                    $and: [{
                                        'contactnumber': {
                                            $elemMatch: {
                                                'value': member.contact,
                                                'verified': true
                                            }
                                        }
                                    }, {
                                        validprofile: true
                                    }]
                                }
                            } else if (member.moc === 'email') {
                                util.getFormattedEmail(member.contact, function(formattedEmail) {
                                    queryToFind = {
                                        $and: [{
                                            'email': {
                                                $elemMatch: {
                                                    'formattedValue': formattedEmail,
                                                    'verified': true
                                                }
                                            }
                                        }, {
                                            validprofile: true
                                        }]
                                    }
                                });
                            }
                            db.profiles.findOne(queryToFind, function(err, memberProfile) {
                                if (err) {
                                    log.error('addMemberToGroup:Error in getprofile method for email ' + emailOfMember + ' ' + err)
                                } else {
                                    var actualCount = member.memberscount
                                    if (member.memberscount == 1) {
                                        actualCount = groupInfo.attendees.length + 1
                                    }

                                    var emailData = {
                                        invitecode: util.getRandomCode(),
                                        moc: member.moc,
                                        invitinguser: usernameOfInvitingPerson.firstname + ' ' + usernameOfInvitingPerson.lastname,
                                        invitinguserfirstname: usernameOfInvitingPerson.firstname,
                                        invitinguserlastname: usernameOfInvitingPerson.lastname,
                                        host: config.HostName,
                                        roomid: roomid,
                                        invitedthrough: groupInfo.purpose,
                                        members: actualCount,
                                        userid: userId,
                                        memberUsername: member.username,
                                        emailtemplate: member.emailtemplate,
                                        summary: groupInfo.message,
                                        grouptitle: groupInfo.groupName,
                                        contact: member.contact,
                                        clienttime: groupInfo.clienttime,
                                        purpose: groupInfo.purpose,
                                        groupname: groupInfo.groupName,
                                        requestId: requestId
                                    }
                                    var query
                                    if (memberProfile == null) {
                                        emailData.existingmember = false
                                        log.info('addMemberToGroup:Member is NOT an chat member' + member.contact)
                                        if (groupInfo.message != '') {
                                            query = {
                                                'userid': '',
                                                'roomId': roomid,
                                                'requestId': requestId,
                                                'jointime': '',
                                                'lastremaindertime': '',
                                                'contact': member.contact,
                                                'moc': member.moc,
                                                'username': member.username,
                                                'unreadcount': 1,
                                                'chatprogress': false,
                                                'accepted': false,
                                                'groupName': groupName
                                            }
                                        } else {
                                            query = {
                                                'userid': '',
                                                'roomId': roomid,
                                                'requestId': requestId,
                                                'jointime': '',
                                                'lastremaindertime': '',
                                                'contact': member.contact,
                                                'moc': member.moc,
                                                'username': member.username,
                                                'unreadcount': 0,
                                                'chatprogress': false,
                                                'accepted': false,
                                                'groupName': groupName
                                            }
                                        }

                                        updateAndMailToMember(emailData, query, member, notifyMail, function(result){
                                          callback(result);
                                        })
                                    } else {
                                        emailData.existingmember = true
                                        log.info('member details' + JSON.stringify(member))
                                        log.info('addMemberToGroup:Member is an chat member : ' + member.contact)
                                            // Push Notify the chat members
                                        if (notifyMail && member.moc === 'email' && memberProfile.status !== 'loggedout') { // Jon wanted have NO notification in case of SMS as MOC
                                            db.chatGroup.aggregate(
                                                [{
                                                    $match: {
                                                        'attendees': {
                                                            $elemMatch: {
                                                                'userid': memberProfile.userId
                                                            }
                                                        }
                                                    }
                                                }, {
                                                    $unwind: '$attendees'
                                                }, {
                                                    $match: {
                                                        'attendees.userid': memberProfile.userId
                                                    }
                                                }, {
                                                    $group: {
                                                        _id: null,
                                                        count: {
                                                            $sum: '$attendees.unreadcount'
                                                        }
                                                    }
                                                }],
                                                function(err, outcome) {
                                                    log.info('Total unread: ' + JSON.stringify(outcome))
                                                    var totalUnread = 0
                                                    if (outcome.length != 0 && outcome[0].count != 'undefined') {
                                                        totalUnread = outcome[0].count + 1
                                                        emailData.allunread = totalUnread
                                                    }
                                                    util.pushNotify(memberProfile.deviceInfo[0].devicetype, memberProfile.deviceInfo[0].devicetoken, roomid, userId, member.memberscount, 'chat', emailData)
                                                })
                                        }
                                        log.info('addMemberToGroup:Existing connections add with userid')
                                        if (groupInfo.message != '') {
                                            query = {
                                                'userid': memberProfile.userId,
                                                'roomId': roomid,
                                                'requestId': requestId,
                                                'jointime': '',
                                                'lastremaindertime': '',
                                                'contact': member.contact,
                                                'moc': member.moc,
                                                'username': memberProfile.username.firstname + ' ' + memberProfile.username.lastname,
                                                'unreadcount': 1,
                                                'chatprogress': false,
                                                'accepted': false,
                                                'profile_id': memberProfile._id,
                                                'groupName': groupName
                                            }
                                        } else {
                                            query = {
                                                'userid': memberProfile.userId,
                                                'roomId': roomid,
                                                'requestId': requestId,
                                                'jointime': '',
                                                'lastremaindertime': '',
                                                'contact': member.contact,
                                                'moc': member.moc,
                                                'username': memberProfile.username.firstname + ' ' + memberProfile.username.lastname,
                                                'unreadcount': 0,
                                                'chatprogress': false,
                                                'accepted': false,
                                                'profile_id': memberProfile._id,
                                                'groupName': groupName
                                            }
                                        }
                                        updateAndMailToMember(emailData, query, member, notifyMail, function(result){
                                          callback(result)
                                        })
                                    }
                                }
                            })
                            log.info(' member added:' + JSON.stringify(member))
                        }
                    }
                })
            } else {
                log.info('User is not an Admin to add member for group:' + roomid)
                callback({
                    'error': 'notanadmin'
                })
            }
        })
    } catch (err) {
        log.error('error in addMemberToGroup:' + err)
    }
}

// Find A group based on the roomID
findGroupForTheRoomId = function(roomid, callback) {
    try {
        log.info('findGroupForTheRoomId :' + roomid)
        db.chatGroup.findOne({
            roomId: roomid
        }).populate('attendees.profile_id', 'profileimage.profileImageUrl skills ').exec(function(err, result) {
            var resultObtained
            if (err) {
                log.info(err)
            } else {
                if (!result) {
                    return callback(emptyString)
                } else {
                    resultObtained = result
                    log.info('resulting group:' + result.roomId)
                    return callback(resultObtained)
                }
            }
        })
    } catch (err) {
        log.error('error in findGroupForTheRoomId:' + err)
    }
}

// Find whether the user is an admin
validateForAdminUser = function(userId, roomid, callback) {
    try {
        log.info('validateForAdminUser for userID: ' + userId + ' and roomID: ' + roomid)
        db.chatGroup.findOne({
            $and: [{
                'roomId': roomid.trim()
            }, {
                'admins': {
                    '$elemMatch': {
                        '$eq': userId.trim()
                    }
                }
            }]
        }, function(err, group) {
            if (err) {
                log.error('validateForAdminUser ADMIN ERROR:')
            } else {
                if (group == null) {
                    log.info('validateForAdminUser NOT AN ADMIN:')
                    callback(false, 0)
                } else {
                    log.info('validateForAdminUser valid ADMIN :')
                    callback(true, group)
                }
            }
        })
    } catch (err) {
        log.error('error in validateForAdminUser:' + err)
    }
}

// Provide chat delta for a userid from the mentioned time
// params: userid,timestamp
module.exports.chatDelta = function(req, res) {
    try {
        log.info('chatDelta request')
        var userId = req.body.userid.trim()
        var timeStamp = req.body.timestamp.trim()
        var resultObtained = []
        var noOfRooms
        var count

        sendResponse = function(counted) {
            log.info('sendResponse :' + counted)
            if (counted == (noOfRooms - 1)) {
                res.json({
                    'history': resultObtained
                })
            }
        }

        if (!userId || !timeStamp) {
            res.status(417).json({
                'error': 'notavalidinput'
            })
            return
        } else {
            // Send the delta of the history
            db.chatGroup.find({
                $and: [{
                    timestamp: {
                        $gte: lastRequestTime
                    }
                }, {
                    'attendees': {
                        $elemMatch: {
                            'userid': req.body.userid
                        }
                    }
                }]
            }, function(err, result) {
                log.info('groups found :' + result.length)
                noOfRooms = result.length
                for (var i in result) {
                    findChatHistoryForARoomId(result[i].roomId, timeStamp, function(history) {
                        if (!history || !Array.isArray(history) || history.length === 0) {
                            resultObtained[count++] = '0'
                        } else {
                            resultObtained[count++] = history
                            sendResponse(count)
                        }
                    })
                }
            })
        }
    } catch (err) {
        log.error('error in validateForAdminUser:' + err)
        res.status(500).json({
            'error': 'error in validateForAdminUser'
        })
    }
}

// Provides chatList(rooms) for a userid
// params: userid
module.exports.chatList = function(req, res) {
    try {
        log.info(' chatList req :' + req.body.userid + 'with timestamp' + req.body.timestamp)
        var userId = req.body.userid.trim()
        var lastRequestTime = req.body.timestamp.trim()
        var time = util.getCurrentTimestamp()
        if (!userId) {
            log.error('error in chatList: notavalidinput')
            res.status(417).json({
                'error': 'notavalidinput'
            })
            return
        }
        db.chatGroup.find({
            $and: [{
                $or: [{
                    'lastChat.timestamp': {
                        $gt: lastRequestTime
                    }
                }, {
                    'attendees': {
                        $elemMatch: {
                            'jointime': {
                                $gt: lastRequestTime
                            }
                        }
                    }
                }]
            }, {
                'attendees': {
                    $elemMatch: {
                        'userid': req.body.userid
                    }
                }
            }]
        }).populate('attendees.profile_id').exec(function(err, result) {
            log.info('groups found :' + result.length)
            res.json({
                'rooms': result,
                'lastsynctime': time
            })
        })
    } catch (err) {
        log.info('error in chatList:' + err)
        res.status(500).json({
            'error': 'error in chatList'
        })
    }
}

module.exports.updateUnreadCountForAnUser = function(req, res) {
    var rooms = []
    var userID = req.body.userid
    rooms = req.body.rooms
    try {
        updateRoom = function(roomInfo, countToUpdate, userID) {
            // log.info(":::updateRoom: "+userID)
            db.chatGroup.update({
                $and: [{
                    roomId: roomInfo.roomid
                }, {
                    'attendees': {
                        $elemMatch: {
                            'userid': userID
                        }
                    }
                }]
            }, {
                $set: {
                    'attendees.$.unreadcount': countToUpdate
                }
            }, function(err, result) {
                if (err) {
                    log.error('error in updating chatRoom count')
                } else {}
            })
        }

        findAttendeeInfo = function(roomInfo, userID) {
            db.chatGroup.aggregate(
                [{
                    $match: {
                        $and: [{
                            'roomId': roomInfo.roomid
                        }, {
                            'attendees': {
                                $elemMatch: {
                                    'userid': userID
                                }
                            }
                        }]
                    }
                }, {
                    $unwind: '$attendees'
                }, {
                    $match: {
                        'attendees.userid': userID
                    }
                }, {
                    $group: {
                        _id: null,
                        count: {
                            '$sum': '$attendees.unreadcount'
                        }
                    }
                }],
                function(err, existingInfo) {
                    var oldCount = 0
                    if (existingInfo.length !== 0 && existingInfo[0].count != 'undefined') {
                        oldCount = existingInfo[0].count
                        updateRoom(roomInfo, oldCount - roomInfo.count, userID)
                    }
                })
        }

        log.info(' updateUnreadCountForAnUser req :' + req.body.userid + 'ROOMS: ' + JSON.stringify(rooms))
        for (i in rooms) {
            // log.info("Every room :"+rooms[i].roomid);
            findAttendeeInfo(rooms[i], userID)
        }
        res.status(200).json({
            'status': 'received rooms for ' + userID
        })
    } catch (err) {
        log.error('error in updateUnreadCountForAnUser:' + err)
        res.status(500).json({
            'error': 'error in updateUnreadCountForAnUser'
        })
    }
}

// Provides chat history based on roomID
// params: roomid
module.exports.chatHistoryForRoomId = function(req, res) {
    try {
        log.info(' chatHistoryForRoomId req :' + req.body.roomid)
        var roomId = req.body.roomid.trim()
        var lastRequestTime = req.body.timestamp

        if (!roomId) {
            log.error('error in chatHistoryForRoomId:notavalidinput')
            res.status(417).json({
                'error': 'notavalidinput'
            })
            return
        }
        findChatHistoryForARoomId(roomId, lastRequestTime, function(response) {
            res.json({
                'history': response
            })
        })
    } catch (err) {
        log.error('error in chatHistoryForRoomId:' + err)
        res.status(500).json({
            'error': 'error in chatHistoryForRoomId'
        })
    }
}

// Find the groups for the provided userName
findGroupsForAUserName = function(uName, callback) {
    try {
        log.info('findGroupsForAUserName')
            // Find the groups for the userName
        db.profiles.find({
            userId: uName
        }, function(err, result) {
            var resultObtained
            if (err) {
                log.info(err)
            } else {
                if (!result || !Array.isArray(result) || result.length === 0) {
                    log.info('findGroupsForAUserName provides NULL:')
                    return callback(emptyString)
                } else {
                    resultObtained = result
                    log.info(resultObtained)
                    return callback(resultObtained.rooms)
                }
            }
        })
    } catch (err) {
        log.error('error in findGroupsForAUserName:')
    }
}

// Find chat history for a Room
findChatHistoryForARoomId = function(rId, lastRequestTime, callback) {
    try {
        log.info('findChatHistoryForARoomId' + rId + 'timestamp:' + lastRequestTime)
            // With the roomid find the chat from Communication logs.
        db.chatlog.find({
            $and: [{
                timestamp: {
                    $gt: lastRequestTime
                }
            }, {
                roomId: rId
            }]
        }, function(err, result) {
            var resultObtained
            if (err) {
                log.info(err)
            } else {
                if (!result || !Array.isArray(result) || result.length === 0) {
                    log.info('findChatHistoryForARoomId NO result')
                    return callback(emptyString)
                } else {
                    log.info('findChatHistoryForARoomId Chats found for roomID: ' + rId)
                    resultObtained = result
                    return callback(resultObtained)
                }
            }
        })
    } catch (err) {
        log.error('error in findChatHistoryForARoomId:')
        res.status(500).json({
            'error': 'error in findChatHistoryForARoomId'
        })
    }
}

createNewSelfRoom = function(userIdOfAttendee, lastmessage, adminRoom, notify) {
    var selfRoomid = userIdOfAttendee;
    var userIdOfAttendee = userIdOfAttendee.replace('self-', '');
    db.profiles.findOne({
        'userId': userIdOfAttendee
    }, function(err, profileFound) {
        if (err) {
            log.info("updateLastChatAndPushNotify-error in finding Profile");
        } else if (profileFound == null || profileFound == undefined) {
            log.info("updateLastChatAndPushNotify- Profile NOT found");
        } else {
            var userName = config.AdminGroupName;
            var contact = profileFound.loginkey;
            var defaultMOC = "sms";
            var timeStamp = util.getCurrentTimestamp();
            var group = new db.chatGroup({
                groupName: adminRoom.groupName,
                roomId: selfRoomid,
                attendees: {
                    'userid': userIdOfAttendee,
                    'requestId': '',
                    'jointime': timeStamp,
                    'lastremaindertime': '',
                    'initialurl': '',
                    'contact': contact,
                    'moc': defaultMOC,
                    'username': userName,
                    'unreadcount': 1,
                    'chatprogress': false,
                    'profile_id': profileFound._id
                },
                attendeesBackUp: {
                    'userid': userIdOfAttendee,
                    'requestId': '',
                    'jointime': timeStamp,
                    'lastremaindertime': '',
                    'initialurl': '',
                    'contact': contact,
                    'moc': defaultMOC,
                    'username': userName,
                    'unreadcount': 1,
                    'chatprogress': false,
                    'profile_id': profileFound._id
                },
                admins: [userIdOfAttendee],
                lastremaindertime: '',
                message: lastmessage.messsage,
                timestamp: timeStamp,
                purpose: ANNOUNCEMENT,
                clienttime: "",
                lastChat: lastmessage
            });
            group.save(function(err, userObj) {
                if (err) {
                    log.error(err);
                } else {
                    log.info('saved initiator details successfully:');
                    var userdetails = {
                        username: userName
                    };
                    var commLogData = {
                        roomId: selfRoomid,
                        from: config.adminUserId,
                        userdetails: userdetails,
                        userid: config.adminUserId,
                        type: 'text',
                        summary: lastmessage.messsage,
                        time: timeStamp
                    };

                    saveSummaryInChatLog(commLogData);
                }
            });
            db.chatGroup.aggregate(
                [{
                    $match: {
                        'attendees': {
                            $elemMatch: {
                                'userid': userIdOfAttendee
                            }
                        }
                    }
                }, {
                    $unwind: "$attendees"
                }, {
                    $match: {
                        'attendees.userid': userIdOfAttendee
                    }
                }, {
                    $group: {
                        _id: null,
                        count: {
                            $sum: '$attendees.unreadcount'
                        }
                    }
                }],
                function(err, outcome) {
                    log.info("Total unread: " + JSON.stringify(outcome));
                    var totalUnread = 0;
                    if (outcome.length != 0 && outcome[0].count != "undefined") {
                        totalUnread = outcome[0].count + 1;
                    }
                    var dataToSend = {
                        fromuserid: lastmessage.userId,
                        touserid: userIdOfAttendee,
                        chattype: lastmessage.chatType,
                        message: lastmessage.messsage,
                        timestamp: lastmessage.time,
                        roomid: selfRoomid,
                        roomunread: "1",
                        allunread: totalUnread
                    }
                    if (notify)
                        util.pushNotify(profileFound.deviceInfo[0].devicetype, profileFound.deviceInfo[0].devicetoken, selfRoomid, userIdOfAttendee, "1", NEWMESSAGE, dataToSend);
                });
        }
    });
}
module.exports.createNewSelfRoom = createNewSelfRoom;
