var db = require('../models/schema');
var log = require('../lib/logger.js');
var smsnumberconfig = require('../config/smsnumberconfig');
var config = require('../config/config');
var twilioInterface = require('twilio');
const SMSLIMIT = 156;
const ATTENDEELIMIT = 25;

module.exports.resetSMSCount = function() {
    try {
        log.info("resetSMSCount method execution begins ...");
        db.smsnumbers.update({
            usageCount: {
                $ne: 0
            }
        }, {
            $set: {
                usageCount: 0,
                availability: true
            }
        }, {
            multi: true
        }, function(err, result) {
            if (err) {
                log.error('error in resetSMSCount:' + err);
            } else {
                log.info("resetSMSCount success :" + JSON.stringify(result));
            }
        });
    } catch (err) {
        log.error("Exception in resetSMSCount:" + err);
    }
}

module.exports.saveNumbers = function() {
    try {
        updateNumbers = function(numberToAdd) {
            log.info("updateNumbers :" + numberToAdd);

            db.smsnumbers.findOne({
                'smsnumber': numberToAdd
            }, function(err, result) {
                if (err) {
                    log.info("updateNumbers :" + err);
                } else if (result != null) {
                    log.info("updateNumber " + numberToAdd + " available");
                } else {
                    log.info("updateNumber " + numberToAdd + " not available:");
                    var smsNumber = new db.smsnumbers({
                        'smsnumber': numberToAdd,
                        'usageCount': 0,
                        'availability': true
                    });
                    smsNumber.save(function(err, userObj) {
                        if (err) {
                            log.error(err);
                        } else {
                            log.info('smsNumber saved successfully:', JSON.stringify(userObj));
                        }
                    });
                }
            });
            // insert the document if it does not exist);
        }
        log.info("saveNumbers execution begins ...");
        var smsNumbersFromConfig = [];
        smsNumbersFromConfig = smsnumberconfig.number;

        for (var i = 0; i < smsNumbersFromConfig.length; i++) {
            updateNumbers(smsNumbersFromConfig[i]);
        }
    } catch (err) {
        log.error("Exception in saveNumbers:" + err);
    }
}

module.exports.sendSMS = function(toNumber, dataToSend, callback) {
    log.info("**********sendSMS method execution parameters*********" + toNumber);
    var accountSid = config.twilioAccountSid;
    var authToken = config.twilioAuthToken;
    var invitedthrough;
    //require the Twilio module and create a REST client
    var client = twilioInterface(accountSid, authToken);
    var formattedString = "You have a chat message. Refer app for further info."
    twilioSMS(client, toNumber, formattedString, "", callback);
}




twilioSMS = function(client, toNumber, formattedString, shortenurl, callback) {

    log.info("Tonumber:" + toNumber);
    log.info("FormattedString:" + formattedString);
    db.smsnumbers.find({
        availability: true
    }, function(err, numberInfo) {
        if (err) {
            log.error("ERROR in getting numbers to send SMS");
        } else if (numberInfo != null && numberInfo.length != 0 && numberInfo != undefined) {
            log.info("found a available number to send SMS");
            //Send SMS to the number selected
            client.messages.create({
                to: toNumber,
                from: numberInfo[0].smsnumber,
                body: formattedString,
                statusCallback: "http://requestb.in/1234abcd"
                    //mediaUrl: "http://farm2.static.flickr.com/1075/1404618563_3ed9a44a3a.jpg",
            }, function(err, message) {
                if (err) {
                    log.info("error in Sending SMS:" + JSON.stringify(err));
                    return 'error';
                } else {
                    message.createdlink = shortenurl;
                    log.info("CREATED LINKS" + message.createdlink);
                    callback(message)
                        //callback({'message status:':'message.status'});
                        //notifyOnError(message.error_code);
                    response = new twilioInterface.TwimlResponse();
                    response.message(message, {
                        statusCallback: message
                    });
                }
            });
            if (numberInfo[0].usageCount == config.smslimit - 1) {
                log.info("Updating the number as FULL");
                db.smsnumbers.update({
                    smsnumber: numberInfo[0].smsnumber
                }, {
                    $set: {
                        'availability': false,
                        'usageCount': numberInfo[0].usageCount + 1
                    }
                }, function(err, updateStatus) {
                    if (err) {
                        log.error("error in updating SMS status:" + err);
                    } else {
                        log.info("updating SMS status:" + JSON.stringify(updateStatus));
                    }

                });
            } else {
                log.info("Updating the number usageCount");
                db.smsnumbers.update({
                    smsnumber: numberInfo[0].smsnumber
                }, {
                    $set: {
                        'usageCount': numberInfo[0].usageCount + 1
                    }
                }, function(err, updateStatus) {
                    if (err) {
                        log.error("error in updating SMS status:" + err);
                    } else {
                        log.info("updating SMS status:" + JSON.stringify(updateStatus));
                    }

                });
            }
        } else {
            callback("nonumbers available");
            log.info("No numbers available to send SMS" + JSON.stringify(numberInfo));
        }
    });

}
