// This is the main file of our Chat app. It initializes a new
// express.js instance, Uses Socket.io for initiating a chat.
// Start the application by running 'node server.js' in terminal

var bodyParser = require('body-parser');
var express = require('express');
var env = require('node-env-file');
env(__dirname + '/../.env');
var winston = require('winston');
var config = require('../config/config');
var logger = require('../lib/logger.js');
var path = require('path');
var schedule = require('node-schedule');
var smsService = require('../smsservice/sms');
var fs = require('fs');
var db = require('../models/schema');
var util = require('../util/util');
function routes(app,io,localClient){
  var cron = config.deleteDeviceSchedule;

  schedule.scheduleJob(cron, function(){
    logger.info('Devil to kill devices!!');
    smsService.resetSMSCount(); // At midnight
  });
  smsService.saveNumbers();

  app.use(bodyParser.urlencoded({ extended: false }))
  // parse application/json
  app.use(bodyParser.json())

  // Everything in public will be accessible from '/'
  app.use('/static',express.static(path.join(__dirname, '/../public')));
  app.use(function (req, res, next) {
    res.setHeader('Access-Control-Allow-Origin', '*' );
    res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');
    res.setHeader('Access-Control-Allow-Headers', 'Content-Type','X-Requested-With');
    res.setHeader('Access-Control-Allow-Credentials', true);
    next();
  });
  app.locals.socketClient = localClient.connect(config.HostName, {reconnect: true});
  app.locals.socketClient.on("connect",function(socketDetails){

  });

  require('../chatComm/chat')(io,logger);
  require('../chatComm/route.js')(app);
  require('../register/route.js')(app);
}
// module.exports = app;
module.exports = logger;
module.exports = routes;
